@@ -23,23 +23,23 @@
  */
 
 #include <config.h>
-
-#include "vino-status-icon.h"
-#include "vino-util.h"
-
 #include <gtk/gtk.h>
 #include <string.h>
-
 #ifdef VINO_ENABLE_LIBNOTIFY
 #include <libnotify/notify.h>
 #endif
 
+#include "vino-status-icon.h"
+#include "vino-enums.h"
+#include "vino-util.h"
+
 struct _VinoStatusIconPrivate
 {
   GtkMenu    *menu;
   VinoServer *server;
   GSList     *clients;
   GtkWidget  *disconnect_dialog;
+  VinoStatusIconVisibility visibility;
 
 #ifdef VINO_ENABLE_LIBNOTIFY
   NotifyNotification *new_client_notification;
@@ -51,7 +51,8 @@ G_DEFINE_TYPE (VinoStatusIcon, vino_status_icon, GTK_TYPE_STATUS_ICON);
 enum
 {
   PROP_0,
-  PROP_SERVER
+  PROP_SERVER,
+  PROP_VISIBILITY
 };
 
 typedef struct
@@ -88,6 +89,40 @@ vino_status_icon_finalize (GObject *object)
   G_OBJECT_CLASS (vino_status_icon_parent_class)->finalize (object);
 }
 
+void
+vino_status_icon_update_state (VinoStatusIcon *icon)
+{
+  char     *tooltip;
+  gboolean visible;
+
+  g_return_if_fail (VINO_IS_STATUS_ICON (icon));
+
+  visible = !vino_server_get_on_hold (icon->priv->server);
+
+  tooltip = g_strdup (_("Desktop sharing is enabled"));
+  
+  if (icon->priv->clients != NULL)
+    {
+      int n_clients;
+
+      n_clients = g_slist_length (icon->priv->clients);
+
+      tooltip = g_strdup_printf (ngettext ("One person is connected",
+                                           "%d people are connected",
+                                           n_clients),
+                                 n_clients);
+      visible = (visible) && ( (icon->priv->visibility == VINO_STATUS_ICON_VISIBILITY_CLIENT) ||
+			     (icon->priv->visibility == VINO_STATUS_ICON_VISIBILITY_ALWAYS) );
+    }
+  else
+    visible = visible && (icon->priv->visibility == VINO_STATUS_ICON_VISIBILITY_ALWAYS);
+
+  gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon), tooltip);
+  gtk_status_icon_set_visible (GTK_STATUS_ICON (icon), visible);
+
+  g_free (tooltip);
+}
+
 static void
 vino_status_icon_init (VinoStatusIcon *icon)
 {
@@ -107,6 +142,9 @@ vino_status_icon_set_property (GObject      *object,
     case PROP_SERVER:
       icon->priv->server = g_value_get_object (value);
       break;
+    case PROP_VISIBILITY:
+      vino_status_icon_set_visibility (icon, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -126,6 +164,9 @@ vino_status_icon_get_property (GObject    *object,
     case PROP_SERVER:
       g_value_set_object (value, icon->priv->server);
       break;
+    case PROP_VISIBILITY:
+      g_value_set_enum (value, icon->priv->visibility);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -140,7 +181,7 @@ vino_status_icon_new (VinoServer *server,
   g_return_val_if_fail (GDK_IS_SCREEN  (screen), NULL);
   
   return g_object_new (VINO_TYPE_STATUS_ICON,
-                       "icon-name", "gnome-remote-desktop",
+                       "icon-name", "preferences-desktop-remote-desktop",
                        "server",    server,
 #if GTK_CHECK_VERSION (2, 11, 0)
                        "screen",    screen,
@@ -253,13 +294,12 @@ vino_status_icon_about (VinoStatusIcon *icon)
     translators = NULL;
 
   gtk_show_about_dialog (NULL,
-                         "name",               _("GNOME Remote Desktop"),
                          "comments",           _("Share your desktop with other users"),
                          "version",            VERSION,
                          "license",            license,
                          "authors",            authors,
                          "translator-credits", translators,
-                         "logo-icon-name",     "gnome-remote-desktop",
+                         "logo-icon-name",     "preferences-desktop-remote-desktop",
                          NULL);
 }
 
@@ -359,6 +399,7 @@ vino_status_icon_popup_menu (GtkStatusIcon *status_icon,
   GtkWidget      *item;
   GSList         *l;
   VinoStatusIconNotify *a;
+  guint          n_clients;
 
   icon->priv->menu = (GtkMenu*) gtk_menu_new ();
 
@@ -374,18 +415,22 @@ vino_status_icon_popup_menu (GtkStatusIcon *status_icon,
   gtk_widget_show (item);
   gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);
 
-  item  = gtk_image_menu_item_new_with_label (_("Disconnect all"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                 gtk_image_new_from_stock (GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU));
+  n_clients = g_slist_length (icon->priv->clients);
+  if (n_clients > 1)
+    {
+      item  = gtk_image_menu_item_new_with_label (_("Disconnect all"));
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+                                     gtk_image_new_from_stock (GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU));
 
-  a = g_new (VinoStatusIconNotify, 1);
-  a->icon   = icon;
-  a->client = NULL;
+      a = g_new (VinoStatusIconNotify, 1);
+      a->icon   = icon;
+      a->client = NULL;
 
-  g_signal_connect_swapped (item, "activate",
-                            G_CALLBACK (vino_status_icon_disconnect_confirm), (gpointer) a);
-  gtk_widget_show (item);
-  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);
+      g_signal_connect_swapped (item, "activate",
+                                G_CALLBACK (vino_status_icon_disconnect_confirm), (gpointer) a);
+      gtk_widget_show (item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);
+    }
 
   for (l = icon->priv->clients; l; l = l->next)
     {
@@ -409,9 +454,12 @@ vino_status_icon_popup_menu (GtkStatusIcon *status_icon,
       g_free (str);
     }
 
-  item = gtk_separator_menu_item_new ();
-  gtk_widget_show (item);
-  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);
+  if (n_clients)
+    {
+      item = gtk_separator_menu_item_new ();
+      gtk_widget_show (item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);
+    }
 
   item = gtk_image_menu_item_new_with_mnemonic (_("_Help"));
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
@@ -442,30 +490,6 @@ vino_status_icon_activate (GtkStatusIcon *icon)
   vino_status_icon_preferences (VINO_STATUS_ICON (icon));
 }
 
-static void
-vino_status_icon_update_tooltip (VinoStatusIcon *icon)
-{
-  char *tooltip;
-
-  tooltip = NULL;
-  
-  if (icon->priv->clients != NULL)
-    {
-      int n_clients;
-
-      n_clients = g_slist_length (icon->priv->clients);
-
-      tooltip = g_strdup_printf (ngettext ("One person is connected",
-                                           "%d people are connected",
-                                           n_clients),
-                                 n_clients);
-    }
-  
-  gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon), tooltip);
-
-  g_free (tooltip);
-}
-
 void
 vino_status_icon_add_client (VinoStatusIcon *icon,
                              VinoClient     *client)
@@ -475,7 +499,7 @@ vino_status_icon_add_client (VinoStatusIcon *icon,
 
   icon->priv->clients = g_slist_append (icon->priv->clients, client);
 
-  vino_status_icon_update_tooltip (icon);
+  vino_status_icon_update_state (icon);
 
   VinoStatusIconNotify *a;
   a = g_new (VinoStatusIconNotify, 1);
@@ -495,7 +519,7 @@ vino_status_icon_remove_client (VinoStatusIcon *icon,
 
   icon->priv->clients = g_slist_remove (icon->priv->clients, client);
 
-  vino_status_icon_update_tooltip (icon);
+  vino_status_icon_update_state (icon);
 
   return icon->priv->clients == NULL;
 }
@@ -524,6 +548,18 @@ vino_status_icon_class_init (VinoStatusIconClass *klass)
 							G_PARAM_STATIC_NAME |
 							G_PARAM_STATIC_NICK |
 							G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (gobject_class,
+				   PROP_VISIBILITY,
+				   g_param_spec_enum ("visibility",
+						      "Icon visibility",
+						      "When the icon must be shown",
+						      VINO_TYPE_STATUS_ICON_VISIBILITY,
+						      VINO_STATUS_ICON_VISIBILITY_CLIENT,
+						      G_PARAM_READWRITE |
+						      G_PARAM_CONSTRUCT |
+						      G_PARAM_STATIC_NAME |
+						      G_PARAM_STATIC_NICK |
+						      G_PARAM_STATIC_BLURB));
 
   g_type_class_add_private (gobject_class, sizeof (VinoStatusIconPrivate));
 }
@@ -532,14 +568,8 @@ vino_status_icon_class_init (VinoStatusIconClass *klass)
 static void
 vino_status_handle_new_client_notification_closed (VinoStatusIcon *icon)
 {
-  /*
-   * FIXME: this looks like a leak, but libnotify crashes
-   *        if we unref here. File a libnotify bug.
-   *
-   * g_object_unref (icon->priv->new_client_notification);
-   */
+  g_object_unref (icon->priv->new_client_notification);
   icon->priv->new_client_notification = NULL;
-  g_print ("Got closed signal\n");
 }
 #endif /* VINO_ENABLE_LIBNOTIFY */
 
@@ -563,7 +593,7 @@ vino_status_icon_show_new_client_notification (gpointer user_data)
     return FALSE;
   }
 
-  if (!notify_is_initted () &&  !notify_init (_("GNOME Remote Desktop")))
+  if (!notify_is_initted () &&  !notify_init (g_get_application_name ()))
     {
       g_printerr (_("Error initializing libnotify\n"));
       g_free (user_data);
@@ -593,7 +623,7 @@ vino_status_icon_show_new_client_notification (gpointer user_data)
   icon->priv->new_client_notification =
     notify_notification_new_with_status_icon (summary,
                                               body,
-                                              "gnome-remote-desktop",
+                                              "preferences-desktop-remote-desktop",
                                               GTK_STATUS_ICON (icon));
 
   g_free (body);
@@ -620,3 +650,25 @@ vino_status_icon_show_new_client_notification (gpointer user_data)
 
   return FALSE;
 }
+
+void
+vino_status_icon_set_visibility (VinoStatusIcon *icon,
+				 VinoStatusIconVisibility  visibility)
+{
+  g_return_if_fail (VINO_IS_STATUS_ICON (icon));
+  g_return_if_fail (visibility != VINO_STATUS_ICON_VISIBILITY_INVALID);
+
+  if (visibility != icon->priv->visibility)
+    {
+      icon->priv->visibility = visibility;
+      vino_status_icon_update_state (icon);
+    }
+}
+
+VinoStatusIconVisibility
+vino_status_icon_get_visibility (VinoStatusIcon *icon)
+{
+  g_return_val_if_fail (VINO_IS_STATUS_ICON (icon), VINO_STATUS_ICON_VISIBILITY_INVALID);
+
+  return icon->priv->visibility;
+}
